-- TobiiWindow.lua
-- Originally TheOneBagWindow.lua written by Rod, extended by Almagnus1

import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";

TobiiWindow = class( Almagnus1.Tobii.ResizeableWindow );

function TobiiWindow:Constructor( _settingsManager )
	Almagnus1.Tobii.ResizeableWindow.Constructor( self );

	self.SettingsManager = _settingsManager;
	self:SetSize(
		self.SettingsManager.Settings.Size.Width,
		self.SettingsManager.Settings.Size.Height );
	self:SetPosition(
		self.SettingsManager.Settings.Location.X,
		self.SettingsManager.Settings.Location.Y ); 
	self:SetVisible( self.SettingsManager.Settings.StartVisibility );
	
	-- Position saving flag
	local hasMoved = false;

	-- Initialize the container that holds the elements referencing the player's inventory.
	self.items = { };
	self.backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();

	self:SetBackColor( Turbine.UI.Color() );
	self:SetOpacity( 1 );
	self:SetWantsKeyEvents( true );

	self:GetItemListBox():SetOrientation( Turbine.UI.Orientation.Horizontal );
	self:GetItemListBox():SetAllowDrop( true );
	
	self.slotsUsed = 0;
	self:SetupBackpack();
	self:PrintTitle();  -- here because we need inventory info first
	
	-- event handlers
	self.PositionChanged = function( sender, args )
		hasMoved = true;
	end

	self.VisibleChanged = function( sender, args )
		if( ( not sender:IsVisible() ) and hasMoved ) then
			self.SettingsManager.Settings.Location.X,
				self.SettingsManager.Settings.Location.Y = self:GetPosition();
			self.SettingsManager:Save();
			hasMoved = false;
		end
	end

	self.Closing = function( sender, args )
		if ( ( not sender:IsVisible() ) and hasMoved ) then
			self.SettingsManager.Settings.Location.X,
				self.SettingsManager.Settings.Location.Y = self:GetPosition();
			self.SettingsManager:Save();
			hasMoved = false;
		end
	end

	self.KeyDown = function( sender, args )
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			sender:SetVisible( false ) 
		end

		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			sender:SetVisible( not sender:IsVisible() ) 
		end
	end

	self.ResizeComplete = function( sender, args )
		self.SettingsManager.Settings.Location.X,
			self.SettingsManager.Settings.Location.Y = self:GetPosition();
		self.SettingsManager.Settings.Size.Width,
			self.SettingsManager.Settings.Size.Height = self:GetSize();
		self.SettingsManager:Save();
	end
	
	self:GetItemListBox().DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();
		if ( shortcut ~= nil ) then
			local destinationItemControl = self:GetItemListBox():GetItemAt( args.X, args.Y );

			-- if nil, then it's out of range, so don't do anything
			if ( destinationItemControl ~= nil ) then
				local destinationIndex =
					self:GetItemListBox():IndexOfItem( destinationItemControl );

				self.backpack:PerformShortcutDrop(
					shortcut,
					destinationIndex,
					Turbine.UI.Control.IsShiftKeyDown() );
			end
		end
	end

	self.backpack.ItemAdded = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		self.slotsUsed = self.slotsUsed + 1;
		self:PrintTitle();
	end

	self.backpack.ItemRemoved = function( sender, args )
		self.items[args.Index]:SetItem( nil );
		self.slotsUsed = self.slotsUsed - 1;
		self:PrintTitle();
	end

	self.backpack.ItemMoved = function( sender, args )
		self.items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		self.items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
	end

end

function TobiiWindow:DisplayCounter( enabled )
	self.SettingsManager.Settings.DisplayCounter = enabled;
	self.SettingsManager:Save();
	
	self:PrintTitle();
end

function TobiiWindow:ShowUsed( toggle )
	self.SettingsManager.Settings.ShowUsed = toggle;
	self.SettingsManager.Settings.DisplayCounter = true;
	self.SettingsManager:Save();
	
	self:PrintTitle();
end

function TobiiWindow:SetDefaultVisibility( visibility )
	self.SettingsManager.Settings.StartVisibility = visibility;
	self.SettingsManager:Save();
end

function TobiiWindow:PrintTitle()
	local displayStr = "Tobii";
	
	if ( self.SettingsManager.Settings.DisplayCounter ) then
		local capacity = self.backpack:GetSize();
		
		displayStr = displayStr .. " (";
		if ( not self.SettingsManager.Settings.ShowUsed ) then
			displayStr = displayStr .. ( capacity - self.slotsUsed );
		else
			displayStr = displayStr .. self.slotsUsed;
		end
		
		displayStr = displayStr .. "/" .. capacity .. " ";
		
		if ( not self.SettingsManager.Settings.ShowUsed ) then
			displayStr = displayStr .. "Free)"
		else
			displayStr = displayStr .. "Used)";
		end
	end
	
	self:SetText( displayStr );
end
--[[ Debugging tools
function TobiiWindow:DumpBackpack()
	local backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	local size = backpack:GetSize();
	local item;
	for i = 1, size, 1 do
		item = backpack:GetItem(i);
		if ( item~=nil ) then
			Turbine.Shell.WriteLine( "Slot " .. i .. ": " .. item:GetName() );
		else
			Turbine.Shell.WriteLine( "Slot " .. i .. ": empty" );
		end
	end
end

function TobiiWindow:DumpItem( index )
	local backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	item = backpack:GetItem(index);
	if ( item~=nil ) then
		Turbine.Shell.WriteLine( "Slot " .. index .. ": " .. item:GetName() );
	else
		Turbine.Shell.WriteLine( "Slot " .. index .. ": empty" );
	end
end
--]]

function TobiiWindow:SetupBackpack()
	if(self:GetItemListBox():GetItemCount() ~= 0) then
		self:GetItemListBox():ClearItems();
	end
	
	local backpackSize = self.backpack:GetSize();
	local temp;
	local item;

	for i = 1, backpackSize, 1 do
		if ( self.items[i] ) then
			self.items[i]:SetParent( nil );
		end
		
		-- TODO: Convert this to a Quickslot, but set changed event handler to boot an alias if one
		--       is ever set.
		-- Get item and set background color
		item = self.backpack:GetItem( i )
		if ( item ~= nil ) then
			self.slotsUsed = self.slotsUsed + 1;
		end
		
		temp = Turbine.UI.Lotro.ItemControl( item );
		temp:SetBackground( "Almagnus1/Tobii/Resources/slotBackground.tga" );
		temp:SetBlendMode( Turbine.UI.BlendMode.Overlay );
		
		self.items[i] = temp;
		self:GetItemListBox():AddItem( self.items[i] );
	end
end
